/**
 * Gymky.me
 * Created by xavi on 07/01/16.
 */

(function(){

    'use strict';
    angular
        .module('games')
        .controller('PlayGameCtrl', [
            '$scope',
            '$state',
            '$stateParams',
            '$cookies',
            '$timeout',
            'Upload',
            'ActivitiesService',
            PlayGameCtrlFn
        ]);

    function PlayGameCtrlFn( $scope, $state, $stateParams, $cookies, $timeout, Upload, ActivitiesSrv ) {

        var self = this;
        self.user = $cookies.getObject( 'currentUser' );
        self.activityId = $stateParams.activityId;

        __checkAuthentication();

        self.success = false;
        self.failed = false;
        self.picFile = null;
        self.gameCompleted = false;
        self.responseHandlers = {

            setTask: function( data ) {

                if ( !data.result ) {

                    var kind_of_task = data.kind_of;

                    __setTaskScope( data );

                    switch (kind_of_task) {
                        case 'Question':
                            __setQuestionTask( data );
                            break;
                        case 'Multiple':
                            __setMultipleTask( data );
                            break;
                        case 'Picture':
                            __setPictureTask( data );
                            break;
                        case 'Position':
                            __setPositionTask( data );
                            break;
                    }

                } else {

                    self.loading = false;
                    self.gameCompleted = true;
                    __getResults();

                }


            },
            checkResponse: function( data ){

                self.loading = false;

                if ( data.response ) {
                    __updateTask( true )

                } else {
                    __updateTask( false )
                }

            },
            setResults: function( data ) {

                self.result = data;
                self.loading = false;

            },
            getErrors: function( data ) {
                self.error = data;
                self.loading = false;
            }

        };

        self.setTask = function() {

            self.loading = true;

            var taskId = ( angular.isDefined( self.task ) ) ? Number( self.task.id ) + 1 : 0;

            var defaultParams = {
                activityId  : self.activityId,
                task        : taskId,
                userId      : self.user.id
            };

            ActivitiesSrv.task.get(
                defaultParams,
                self.responseHandlers.setTask,
                self.responseHandlers.getErrors
            );

        };

        self.checkResponse = function( response ) {

            self.loading = true;

            var defaultParams = {
                activityId  : self.activityId,
                task        : self.task.id,
                response    : response,
                userId      : self.user.id
            };

            ActivitiesSrv.question.get(
                defaultParams,
                self.responseHandlers.checkResponse,
                self.responseHandlers.getErrors
            )

        };

        self.uploadBackground = function( image ) {

            if ( angular.isArray( image ) ) image = image[0];

            if ( image.type !== 'image/png' && image.type !== 'image/jpeg' ) {
                var errorResponse = 'Only PNG and JPEG are accepted.';
                self.responseHandlers.getError( errorResponse )
            }

            self.loading = true;
            self.uploadInProgress = true;

            var activityId = $stateParams.activityId;
            var task = self.task.id;
            var userId = self.user.id;
            var defaultParams = {
                url: 'api/activities/background/' + activityId + '/' + task + '/' + userId,
                method: 'PUT',
                data: { file : image }
            };

            Upload.upload( defaultParams )
                .success(function( data ){
                    self.responseHandlers.checkResponse(data);
                })
                .error(function(err){
                    self.uploadInProgress = false;
                    console.log('Error uploading file: ' + err.message || err);
                }
            );

        };

        function __getResults() {

            self.loading = true;

            var defaultParams = {
                activityId  : self.activityId,
                userId      : self.user.id
            };

            ActivitiesSrv.results.query(
                defaultParams,
                self.responseHandlers.setResults,
                self.responseHandlers.getErrors
            )

        }

        function __updateTask( state ) {

            if ( self.picFile ) self.picFile === null;

            $timeout(function(){
                if ( state ) {
                    self.success = false;
                }else{
                    self.failed = false;
                }
            }, 1500);
            self.setTask();
        }

        function __setQuestionTask( task ){

            self.response = undefined;
            self.task = task;
            self.loading = false;

        }

        /**
         * Set Multiple Question Data
         * @param task
         * @private
         */
        function __setMultipleTask( task ) {

            var responses = [];

            for ( var i=0; i < task.responses.length; i++ ){
                responses.push( task.responses[i].value );
                responses.sort();
            }

            self.responses = responses;
            self.task = task;
            self.loading = false;

        }

        function __setPictureTask( task ){

            self.picFile = undefined;
            self.task = task;
            self.loading = false;

        }

        function __setPositionTask( task ){

            self.task = task;
            self.loading = false;

        }

        /**
         * Set Directives Scope
         * @param data
         * @private
         */
        function __setTaskScope( data ) {

            var task = [];

            $scope.id = Number( data.id );
            $scope.gameTitle = self.user.gameTitle;

            for (var i = 0; i < data.total; i ++){

                task.push( { taskId : i, icon: data.kind_of } );
                $scope.total = task;

            }

        }

        /**
         * Check User Authentication
         * @private
         */
        function __checkAuthentication(){

            if ( angular.isUndefined( self.user ) ||
                self.user.gameId !== self.activityId  ) {

                $state.go( 'login' );

            }

        }

    }

})();
