/**
 * Gymky.me
 * Created by xavi on 27/12/15.
 */

(function(){

    'use strict';
    angular
        .module('games')
        .controller('GameController', [
            '$stateParams',
            '$state',
            '$timeout',
            '$cookies',
            'ActivitiesService',
            GameCtrlFn
        ]);

    function GameCtrlFn( $stateParams, $state, $timeout, $cookies, ActivitiesSrv ){

        var self = this;
        var error = 'Sorry but you can not access to this game!';
        var notAvailable = 'This game is not yet available, check your notifications to make sure the right game date.';

        self.loading = true;
        self.success = false;
        self.responseHandlers = {

            getActivity: function( data ){

                var formatDate = new Date( data.date );

                if( new Date() >= formatDate ) {
                    self.available = true;
                    self.activity = data;
                    self.loading = false;
                }else{
                    self.message = notAvailable;
                    self.available = false;
                    self.loading = false;
                }

            },
            unknownActivity: function(){

                $state.go('login');

            },
            goToPlay: function( data ){

                if ( data.result ) {

                    var currentUser = undefined;

                    if ( data.team.email === self.email ){

                        currentUser = {
                            id      : data.team._id,
                            email   : data.team.email,
                            leader  : data.team.leader,
                            title   : data.team.title,
                            gameId  : $stateParams.activityId,
                            gameTitle : self.activity.title
                        };

                        console.log( currentUser );

                        $cookies.putObject( 'currentUser', currentUser );

                        $timeout(function(){
                            $state.go('game.introduction', { activityId : $stateParams.activityId });
                        },1500);

                    }

                }else{
                    self.responseHandlers.unknownLeader();
                    self.loading = false;
                }


            },
            unknownLeader: function(){

                __cleanFields();
                self.loading = false;
                self.error = error;

            }

        };

        self.findActivity = function() {

            if( angular.isDefined( $stateParams ) ) {

                var defaultParams = { activityId : $stateParams.activityId };

                ActivitiesSrv.team.get(
                    defaultParams,
                    self.responseHandlers.getActivity,
                    self.responseHandlers.unknownActivity
                );

            }

        };

        self.checkEmail = function(){

            if ( self.email ) {

                self.loading = true;

                var defaultParams = {
                    activityId  : $stateParams.activityId,
                    leaderEmail : self.email.trim()
                };

                ActivitiesSrv.game.get(
                    defaultParams,
                    self.responseHandlers.goToPlay,
                    self.responseHandlers.unknownLeader
                );

            }

        };

        function __cleanFields(){

            self.email = undefined;

        }

        $(window).scroll(function () {
            var oVal;
            oVal = $(window).scrollTop() / 200;
            return $('.blur').css('opacity', oVal);
        });

    }

})();
