/**
 * Gymky.me
 * Created by xavi on 15/11/15.
 */

(function() {

    'use strict';
    angular
        .module('profile')
        .controller('ProfileController', [
            '$stateParams',
            '$state',
            '$timeout',
            'Upload',
            'Authentication',
            'ProfileService',
            ProfileCtrlFn
        ]);

    function ProfileCtrlFn($stateParams, $state, $timeout, Upload, AuthenticationSrv, ProfileSrv ){

        var self = this;
        var genericError = 'Ups something was wrong!';

        self.authentication = AuthenticationSrv;
        self.loading = true;

        __checkAuthentication();

        self.success = false;
        self.params = $state.params;

        /**
         * Handle API Rest responses
         * @type {{getProfile: Function, getError: Function, uploadAvatar: Function}}
         */
        self.responseHandlers = {

            getProfile: function() {

                self.loading = false;
                self.success = true;

                $timeout( function(){ $state.go('login') }, 1500 );

            },
            setProfile: function (data){

                self.loading = false;
                self.user = data;
            },
            getError: function( errorResponse ) {

                self.error = errorResponse;
                self.looding = false;

            },
            uploadAvatar: function(){

                self.loading = false;

            }

        };

        self.uploadAvatar = function( image ) {

            if ( angular.isArray( image ) ) image = image[0];

            if ( image.type !== 'image/png' && image.type !== 'image/jpeg' ) {
                var errorResponse = 'Only PNG and JPEG are accepted.';
                self.responseHandlers.getError( errorResponse )
            }

            self.loading = true;
            self.uploadInProgress = true;
            self.uploadProgress = 0;

            var userId = self.params.userId;
            var defaultParams = {
                url: 'api/users/' + userId + '/avatar',
                method: 'PUT',
                data: { file : image }
            };

            Upload.upload( defaultParams )
                .success(function( data ){
                    self.responseHandlers.uploadAvatar();
                })
                .error(function(err){
                    self.uploadInProgress = false;
                    console.log('Error uploading file: ' + err.message || err);
                }
            );

        };

        self.getProfile = function() {

            var userId = { userId: $stateParams.userId }

            ProfileSrv.profile.get(
                userId,
                self.responseHandlers.setProfile,
                self.responseHandlers.getError
            );

        };

        self.updateProfile = function() {

            self.loading = true;
            self.user.$update(function() {

                self.getProfile(
                    { userId: $stateParams.userId },
                    self.responseHandlers.getProfile(),
                    self.responseHandlers.getError( genericError )
                )

            }, function(errorResponse) {
                self.error = errorResponse.data.message;
                self.loading = false;
            });

        };

        self.cancelEditProfile = function(){

            $state.go('login');

        };

        /**
         * Check User Authentication
         * @private
         */
        function __checkAuthentication() {

            if (self.authentication.user === null) $state.go('login');

        }

    }


})();
